--[[
Copyright (c) 2010 Matthias Richter

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Except as contained in this notice, the name(s) of the above copyright holders
shall not be used in advertising or otherwise to promote the sale, use or
other dealings in this Software without prior written authorization.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
]]--

require 'vector'

local camera = {}
camera.__index = camera
function Camera(pos, zoom, rot)
	local pos  = pos or vector(love.graphics.getWidth(), love.graphics.getHeight()) / 2
	local zoom = zoom or 1
	local rot  = rot or 0
	return setmetatable({pos = pos, zoom = zoom, rot = rot}, camera)
end

function camera:setLimits(left,top,right,bottom)
	self.limits = {left or 0,top or 0,right or love.graphics.getWidth(),bottom or love.graphics.getHeight()}
end
function camera:getLimits()
	return self.limits
end

function camera:rotate(phi)
	self.rot = self.rot + phi
end

function camera:translate(t)
	self.pos = self.pos + t
end

function camera:predraw()
	local center = vector(love.graphics.getWidth(), love.graphics.getHeight()) / (self.zoom * 2)
	local x,y = self.pos:unpack()
	love.graphics.push()
	love.graphics.scale(self.zoom)
	love.graphics.translate(center:unpack())
	love.graphics.rotate(self.rot)
  	if self.limits then
		if     x < self.limits[1] then x = self.limits[1]     --left
		elseif x > self.limits[3] then x = self.limits[3] end --right
		if     y < self.limits[2] then y = self.limits[2]     --top
		elseif y > self.limits[4] then y = self.limits[4] end --bottom
  	end

	love.graphics.translate(-x,-y)
	--love.graphics.translate((-self.pos):unpack())
end

function camera:postdraw()
	love.graphics.pop()
end

function camera:draw(func)
	self:predraw()
	func()
	self:postdraw()
end

function camera:toCameraCoords(p)
	local w,h = love.graphics.getWidth(), love.graphics.getHeight()
	local p = (p - self.pos):rotate_inplace(self.rot)
	return vector(p.x * self.zoom + w/2, p.y * self.zoom + h/2)
end

function camera:toWorldCoords(p)
	local w,h = love.graphics.getWidth(), love.graphics.getHeight()
	p = vector((p.x-w/2) / self.zoom, (p.y-h/2) / self.zoom):rotate_inplace(-self.rot)
	if self.limits then
		local x,y = self.pos:unpack()
 		if     x < self.limits[1] then x = self.limits[1]  --left
 		elseif x > self.limits[3] then x = self.limits[3] end --right
 		if     y < self.limits[2] then y = self.limits[2]  --top
 		elseif y > self.limits[4] then y = self.limits[4] end --bottom
		return p + vector(x,y)
	else
		return p + self.pos
	end
end

function camera:mousepos()
	return self:toWorldCoords(vector(love.mouse.getPosition()))
end
