require 'gamestate'
require 'camera'
require 'timer'

function bg_f(x, y, r, g, b, a)
    r=40
    g=23
    b=96
    a = 255
    if (math.fmod(x,32) == 0) then
        r=r+32
        g=g+32
        b=b+32
    end
    if (math.fmod(y,32) == 0) then
        r=r+32
        g=g+32
        b=b+32
    end
    return r,g,b,a
end

function get_object_at(x,y)
	for index,entity in ipairs(level) do
		if entity.shape:testPoint(x,y) then
			return entity,index
		end
	end
	return nil
end

game = Gamestate.new()
function game:enter(from,name)
    self.name = name
    level = {}
    actors = {}
    for_delete = {}
	help = true
	help_text = {}
    print("Entering level",name, "from",from)
    print("Loading level",name)
    init = love.filesystem.load("levels/"..name)
    init()
	--  
	-- player2 = actors.pony2
	-- TODO:     (, ) 
	frames = 0
	fps = 0
	Timer.addPeriodic(1, function() fps, frames = frames, 0  end)
	--only for debug
--	if DEBUG then
		idata = love.image.newImageData(128, 128)
		idata:mapPixel(bg_f)
		bg = love.graphics.newImage(idata)
		bg:setWrap("repeat", "repeat")
		bgq = love.graphics.newQuad(world_size[1],world_size[2],world_size[3],world_size[4],128,128)
--	end
	--TODO: foreground and background
	--foreground = love.graphics.newFramebuffer()
	--background = love.graphics.newFramebuffer()
	--hud = love.graphics.newFramebuffer()
end

function game:leave()
    print("Leaving level",self.name)
    for _, object in pairs(level) do
        object:preremove()
    end
    world:update(1/60)
    for _, object in pairs(level) do
        object:destroy()
    end
	Timer.clear()
    world = nil
    level = nil
    actors = nil
    for_delete = nil
    player = nil
    j = nil
	PAUSED = false
end

function game:update(dt)
    if not PAUSED then
        --for _, actor in ipairs(actors) do
            --actor:update(dt)
        --end
        --cursor:update()
        --preremove dynamic objects
        for _, object in pairs(for_delete) do
            object:preremove()
        end
        --update all
        for _, object in pairs(level) do
            object:update(dt)
        end
        world:update(dt)
        if j then
            j:setTarget(camera:mousepos():unpack())
        end
        --remove dynamic objects
        if for_delete ~= {} then
            for index, object in pairs(for_delete) do
                object:destroy()
				print("removing from level #",index)
                table.remove(level,index)
				if actors[index] then print("removing from actors #",index); table.remove(actors,index) end
            end
            for_delete = {}
        end
    end
    --hud:update()
	frames = frames + 1
	Timer.update(dt)
end

function game:draw()
	camera:predraw()
	--love.graphics.setRenderTarget(background)
	--if DEBUG then
	love.graphics.drawq(bg, bgq, 0,0)
	--end
	--love.graphics.setRenderTarget(foreground)
    for _, entity in pairs(level) do
        entity:draw()
    end
	camera:postdraw()
	--love.graphics.setRenderTarget(hud)
	if PAUSED then
		love.graphics.setColor(127,127,127,127)
		love.graphics.rectangle("fill", 10,10, 780,580)
		love.graphics.setColor(255,255,255,255)
		love.graphics.print("P A U S E D",300,260,0,2,2)
	end
	if help then
		love.graphics.setColor(255,255,255,127)
		for i,t in ipairs(help_text) do
			love.graphics.print(t,200,i*20)
		end
	end
	love.graphics.setColor(255,255,255,255)
	love.graphics.print("FPS:"..fps,20,20)
	--love.graphics.setRenderTarget()
	--love.graphics.draw(background)
	--love.graphics.draw(foreground)
	--love.graphics.draw(hud)
end

function game:mousepressed(x,y,button)
	x,y = camera:toWorldCoords(vector(x,y)):unpack()
    if button == "l" then
        if not j then
			object,index = get_object_at(x,y)
			if object and object ~= player then -- ,   
				print("Grab object",index)
				j=love.physics.newMouseJoint(object.body,x,y)
				return
			end
            if not PAUSED then table.insert(level, DynamicCircle(x,y,20)) end
        end
    end
	if button == "m" then
		table.insert(level, DynamicBox(x,y,40))
	end
    if button == "r" then
		object,index = get_object_at(x,y)
		if object then table.insert(for_delete, index, object) end
    end
    if button == "wu" and camera.zoom < 4 then
        camera.zoom = camera.zoom * 1.1
    end
    if button == "wd" and camera.zoom > 0.5 then
        camera.zoom = camera.zoom / 1.1
    end
end

function game:mousereleased(x,y,button)
    if j and button == "l" then
        j:destroy()
        j = false
    end
end

function game:keypressed(key,unicode)
    if key == "escape" then
		if PAUSED then
			Gamestate.switch(menu)
		else
			PAUSED = true
		end
    end
    if key == "p" then
        PAUSED = not PAUSED
    end
    if key == "h" then
        help = not help
    end

    if key == "d" then --dump
		player:dump()
    end
    --TODO: use object methods instead of direct member modification - FIXED
    if key == KEY_UP then
		player:up(true)
    end
    if key == KEY_DOWN then
		player:down(true)
    end
    if key == KEY_LEFT then
        player:left(true)
    end
    if key == KEY_RIGHT then
        player:right(true)
    end
    if key == ' ' then
        player:grab()
    end
	if key == 'tab' then
		player:stop()
		player.controlled = false
		current, player = next(actors,current)
		if not player then current,player = next(actors) end
		player.controlled = true
		camera.pos = player.position
	end
--~     if key == '1' then
--~         player.controlled = false
--~         player = actors[1]
--~ 		camera.pos = player.position
--~         player.controlled = true
--~     end
--~     if key == '2' then
--~         player.controlled = false
--~         player = actors[2]
--~ 		camera.pos = player.position
--~         player.controlled = true
--~     end
--~     if key == '3' then
--~         player.controlled = false
--~         player = actors[3]
--~ 		camera.pos = player.position
--~         player.controlled = true
--~     end
end

function game:keyreleased(key,unicode)
    if key == KEY_UP then
		player:up(false)
    end
    if key == KEY_DOWN then
		player:down(false)
    end
    if key == KEY_LEFT then
        player:left(false)
    end
    if key == KEY_RIGHT then
        player:right(false)
    end
end
function game:focus(f)
	if not f then
		print("lost focus")
		PAUSED = true
	else
		print("gained focus")
	end
end
