-- Shapes categories
GROUND = 1
ACTOR = 2
ENEMY = 3
OTHER = 16
--Callbacks
--Every object with physics must have self-link in shapes data.
function cb_collide(object1, object2, collision)
    --print(a," collding with ",b," at speed ",coll:getVelocity())
    if object1 and object1.cb_collide then
        object1:cb_collide(object2,collision)
    end
    if object2 and object2.cb_collide then
        object2:cb_collide(object1,collision)
    end
end

function cb_persist(object1, object2, collision)
    --print(a," touching ",b)
    if object1 and object1.cb_persist then
        object1:cb_persist(object2,collision)
    end
    if object2 and object2.cb_persist then
        object2:cb_persist(object1,collision)
    end
end

function cb_remove(object1, object2, collision)
    --print(a," uncolliding ",b)
    if object1 and object1.cb_remove then
        object1:cb_remove(object2,collision)
    end
    if object2 and object2.cb_remove then
        object2:cb_remove(object1,collision)
    end
end

function cb_result(a, b, coll)
    --print(a," hit ",b,"resulting with ",coll:getNormal())
    if object1 and object1.cb_result then
        object1:cb_result(object2,collision)
    end
    if object2 and object2.cb_result then
        object2:cb_result(object1,collision)
    end
end

--Misc
DynamicBody = class('DynamicBody')

DynamicBox = class('DynamicBox',DynamicBody)
function DynamicBox:initialize(x,y,width,height)
    --super:initialize(x,y)
    --self.x = x
    --self.y = y
    self.id = global_increment_id()
	self.position = vector(x,y)
    width = width or 20
    height = height or width
    self.body = love.physics.newBody(world, x, y)
    self.shape = love.physics.newRectangleShape( self.body, 0, 0, width, height )
    self.shape:setCategory(OTHER)
	self.shape:setFriction(10)
    self.shape:setData(self)
    self.body:setMassFromShapes()
end

function DynamicBox:update()
	self.position.x, self.position.y = self.body:getPosition()
end

function DynamicBox:draw()
    if DEBUG then
        if self.body and self.shape then
        if self.body:isSleeping() then love.graphics.setColor(0,127,127) else love.graphics.setColor(0,255,255) end
        love.graphics.polygon("line",self.shape:getPoints())
        end
    end
end
function DynamicBox:preremove()
	if DEBUG then print("Prepare for remove",self, self.id) end
    self.shape:setMask(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16)
    self.preremoved = true
end
function DynamicBox:destroy()
    if self.preremoved then
		if DEBUG then print("Removing",self, self.id) end
        self.shape:destroy()
        --self.shape = nil
        self.body:destroy()
        --self.body = nil
        --self = nil
    end
end

DynamicCircle = class('DynamicCircle',DynamicBody)
function DynamicCircle:initialize(x,y,radius)
    self.id = global_increment_id()
	self.position = vector(x,y)
    width = width or 20
    height = height or width
    self.body = love.physics.newBody(world, x, y)
    self.shape = love.physics.newCircleShape( self.body, 0, 0, radius)
    self.shape:setCategory(OTHER)
	--self.shape:setFriction(10)
    self.shape:setData(self)
    self.body:setMassFromShapes()
end

function DynamicCircle:update()
	self.position.x, self.position.y = self.body:getPosition()
end

function DynamicCircle:draw()
    if DEBUG then
        if self.body and self.shape then
        if self.body:isSleeping() then love.graphics.setColor(0,127,127) else love.graphics.setColor(0,255,255) end
        love.graphics.circle("line",self.position.x,self.position.y,self.shape:getRadius())
        end
    end
end
function DynamicCircle:preremove()
	if DEBUG then print("Prepare for remove",self, self.id) end
    self.shape:setMask(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16)
    self.preremoved = true
end
function DynamicCircle:destroy()
    if self.preremoved then
		if DEBUG then print("Removing",self, self.id) end
        self.shape:destroy()
        --self.shape = nil
        self.body:destroy()
        --self.body = nil
        --self = nil
    end
end

--mouse cursor
MouseCursor = class('MouseCursor')
function MouseCursor:initialize()
    self.x = 0
    self.y = 0
    self.body = love.physics.newBody(world, 0, 0)
    self.shape = love.physics.newCircleShape( self.body, 0, 0, 5 )
    self.shape:setSensor(true)
    self.shape:setData(self)
end
function MouseCursor:update()
    self.x, self.y = love.mouse.getPosition()
    self.body:setPosition(self.x,self.y)
end
function MouseCursor:draw()
    love.graphics.setColor(127,127,127)
    love.graphics.circle("line",self.x,self.y,5)
end
function MouseCursor:cb_collide(object)
    print("Mouse over",object)
end
function MouseCursor:cb_remove(object)
    print("Mouse out",object)
end
