Platform = class('Platform')
function Platform:update()
end
function Platform:preremove()
	if DEBUG then print("Preparing for remove",self,self.id) end
    self.shape:setMask(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16)
    self.preremoved = true
end
function Platform:destroy()
    if self.preremoved then
        if DEBUG then print("Removing",self,self.id) end
        self.shape:destroy()
        self.body:destroy()
    end
end

RectanglePlatform = class('RectanglePlatform',Platform)
function RectanglePlatform:initialize(x,y,width,height)
    -- x,y - center position
    self.id = global_increment_id()
    self.width = width
    self.height = height
    self.body = love.physics.newBody( world, x,y )
    self.shape = love.physics.newRectangleShape( self.body, 0, 0, width, height )
    self.shape:setCategory(GROUND)
    self.shape:setData(self)
end
function RectanglePlatform:draw()
    if DEBUG then
        love.graphics.setColor(127,127,127)
        love.graphics.polygon("line",self.shape:getPoints())
    end
end

PolygonPlatform = class('PolygonPlatform',Platform)
function PolygonPlatform:initialize(x,y,...)
    self.id = global_increment_id()
    self.body = love.physics.newBody( world, x,y )
    self.shape = love.physics.newPolygonShape( self.body, ... )
    self.shape:setCategory(GROUND)
    self.shape:setData(self)
end
function PolygonPlatform:draw()
    if DEBUG then
        love.graphics.setColor(127,127,127)
        love.graphics.polygon("line",self.shape:getPoints())
    end
end

RoundedPlatform = class('RoundedPlatform',Platform)
function RoundedPlatform:initialize(x,y,width,height)
    self.id = global_increment_id()
    local radius = height/2
    self.body = love.physics.newBody( world, x,y )
    self.shapes = {}
    table.insert(self.shapes,love.physics.newRectangleShape( self.body, 0, 0, width-height, height ))
    table.insert(self.shapes,love.physics.newCircleShape( self.body, -width/2 + radius, 0, radius ))
    table.insert(self.shapes,love.physics.newCircleShape( self.body, width/2 - radius, 0, radius ))
    for _,shape in ipairs(self.shapes) do
        shape:setCategory(GROUND)
        shape:setData(self)
    end
end
function RoundedPlatform:draw()
    if DEBUG then
        love.graphics.setColor(127,127,127)
        love.graphics.polygon("line",self.shapes.center:getPoints())
        local x,y = self.shapes.left:getWorldCenter()
        local r = self.shapes.left:getRadius()
        love.graphics.circle("line",x,y,r)
        local x,y = self.shapes.right:getWorldCenter()
        local r = self.shapes.right:getRadius()
        love.graphics.circle("line",x,y,r)
    end
end
function RoundedPlatform:preremove()
    for _,shape in ipairs(self.shapes) do
        shape:setMask(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16)
    end
    self.preremoved = true
end
function RoundedPlatform:destroy()
    if self.preremoved then
        if DEBUG then print("Removing",self,self.id) end
        for _,shape in ipairs(self.shapes) do
            shape:destroy()
        end
        self.body:destroy()
    end
end

RotatingPlatform = class('RotatingPlatform',Platform)
function RotatingPlatform:initialize(x,y,width,height, friction, anchor_x, anchor_y)
    self.id = global_increment_id()
    friction = friction or 1000
    anchor_x = anchor_x or x
    anchor_y = anchor_y or y
    self.anchor_body = love.physics.newBody(world, anchor_x, anchor_y)
    self.body = love.physics.newBody(world, x, y)
    self.shape = love.physics.newRectangleShape( self.body, 0, 0, width, height )
    self.shape:setCategory(GROUND)
    self.shape:setData(self)
    self.body:setMassFromShapes()
    self.joint = love.physics.newRevoluteJoint( self.anchor_body, self.body, anchor_x, anchor_y )
    self.joint:setMaxMotorTorque(friction)
    self.joint:setMotorEnabled(true)
end
function RotatingPlatform:draw()
    if DEBUG then
        if self.body:isSleeping() then love.graphics.setColor(127,127,127) else love.graphics.setColor(255,255,255) end
        love.graphics.polygon("line",self.shape:getPoints())
    end
end
function RotatingPlatform:destroy()
    if self.preremoved then
        if DEBUG then print("Removing",self,self.id) end
        self.shape:destroy()
        self.joint:destroy()
        self.body:destroy()
        self.anchor_body:destroy()
    end
end

--Lift = class('Lift',Platform)
--function Lift:initialize(x1,y2,x2,y2)

--end
